<?php

namespace Renfos\Timesheet\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Renfos\Timesheet\Http\Requests\BoardRequest;
use Renfos\Timesheet\Interfaces\BoardRepositoryInterface;
use Renfos\Timesheet\Models\Board;

/**
 * Resource controller class for board.
 */
class BoardResourceController extends BaseController
{

    /**
     * Initialize board resource controller.
     *
     * @param type BoardRepositoryInterface $board
     *
     * @return null
     */
    public function __construct(BoardRepositoryInterface $board)
    {
        parent::__construct();
        $this->repository = $board;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Renfos\Timesheet\Repositories\Criteria\BoardResourceCriteria::class);
    }

    /**
     * Display a list of board.
     *
     * @return Response
     */
    public function index(BoardRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Renfos\Timesheet\Repositories\Presenter\BoardPresenter::class)
                ->$function();
        }

        $boards = $this->repository->paginate();

        return $this->response->title(trans('timesheet::board.names'))
            ->view('timesheet::board.index', true)
            ->data(compact('boards'))
            ->output();
    }

    /**
     * Display board.
     *
     * @param Request $request
     * @param Model   $board
     *
     * @return Response
     */
    public function show(BoardRequest $request, Board $board)
    {

        if ($board->exists) {
            $view = 'timesheet::board.show';
        } else {
            $view = 'timesheet::board.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('timesheet::board.name'))
            ->data(compact('board'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new board.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BoardRequest $request)
    {

        $board = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('timesheet::board.name')) 
            ->view('timesheet::board.create', true) 
            ->data(compact('board'))
            ->output();
    }

    /**
     * Create new board.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BoardRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $board                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('timesheet::board.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('timesheet/board/' . $board->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/timesheet/board'))
                ->redirect();
        }

    }

    /**
     * Show board for editing.
     *
     * @param Request $request
     * @param Model   $board
     *
     * @return Response
     */
    public function edit(BoardRequest $request, Board $board)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('timesheet::board.name'))
            ->view('timesheet::board.edit', true)
            ->data(compact('board'))
            ->output();
    }

    /**
     * Update the board.
     *
     * @param Request $request
     * @param Model   $board
     *
     * @return Response
     */
    public function update(BoardRequest $request, Board $board)
    {
        try {
            $attributes = $request->all();

            $board->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('timesheet::board.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('timesheet/board/' . $board->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('timesheet/board/' . $board->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the board.
     *
     * @param Model   $board
     *
     * @return Response
     */
    public function destroy(BoardRequest $request, Board $board)
    {
        try {

            $board->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('timesheet::board.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('timesheet/board/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('timesheet/board/' . $board->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple board.
     *
     * @param Model   $board
     *
     * @return Response
     */
    public function delete(BoardRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('timesheet::board.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('timesheet/board'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/timesheet/board'))
                ->redirect();
        }

    }

    /**
     * Restore deleted boards.
     *
     * @param Model   $board
     *
     * @return Response
     */
    public function restore(BoardRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('timesheet::board.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/timesheet/board'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/timesheet/board/'))
                ->redirect();
        }

    }

}
