<?php

namespace Renfos\Timesheet\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Renfos\Timesheet\Http\Requests\TaskRequest;
use Renfos\Timesheet\Interfaces\TaskRepositoryInterface;
use Renfos\Timesheet\Models\Task;

/**
 * Resource controller class for task.
 */
class TaskResourceController extends BaseController
{

    /**
     * Initialize task resource controller.
     *
     * @param type TaskRepositoryInterface $task
     *
     * @return null
     */
    public function __construct(TaskRepositoryInterface $task)
    {
        parent::__construct();
        $this->repository = $task;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Renfos\Timesheet\Repositories\Criteria\TaskResourceCriteria::class);
    }

    /**
     * Display a list of task.
     *
     * @return Response
     */
    public function index(TaskRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Renfos\Timesheet\Repositories\Presenter\TaskPresenter::class)
                ->$function();
        }

        $tasks = $this->repository->paginate();

        return $this->response->title(trans('timesheet::task.names'))
            ->view('timesheet::task.index', true)
            ->data(compact('tasks'))
            ->output();
    }

    /**
     * Display task.
     *
     * @param Request $request
     * @param Model   $task
     *
     * @return Response
     */
    public function show(TaskRequest $request, Task $task)
    {

        if ($task->exists) {
            $view = 'timesheet::task.show';
        } else {
            $view = 'timesheet::task.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('timesheet::task.name'))
            ->data(compact('task'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new task.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TaskRequest $request)
    {

        $task = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('timesheet::task.name')) 
            ->view('timesheet::task.create', true) 
            ->data(compact('task'))
            ->output();
    }

    /**
     * Create new task.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TaskRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $task                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('timesheet::task.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('timesheet/task/' . $task->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/timesheet/task'))
                ->redirect();
        }

    }

    /**
     * Show task for editing.
     *
     * @param Request $request
     * @param Model   $task
     *
     * @return Response
     */
    public function edit(TaskRequest $request, Task $task)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('timesheet::task.name'))
            ->view('timesheet::task.edit', true)
            ->data(compact('task'))
            ->output();
    }

    /**
     * Update the task.
     *
     * @param Request $request
     * @param Model   $task
     *
     * @return Response
     */
    public function update(TaskRequest $request, Task $task)
    {
        try {
            $attributes = $request->all();

            $task->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('timesheet::task.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('timesheet/task/' . $task->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('timesheet/task/' . $task->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the task.
     *
     * @param Model   $task
     *
     * @return Response
     */
    public function destroy(TaskRequest $request, Task $task)
    {
        try {

            $task->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('timesheet::task.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('timesheet/task/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('timesheet/task/' . $task->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple task.
     *
     * @param Model   $task
     *
     * @return Response
     */
    public function delete(TaskRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('timesheet::task.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('timesheet/task'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/timesheet/task'))
                ->redirect();
        }

    }

    /**
     * Restore deleted tasks.
     *
     * @param Model   $task
     *
     * @return Response
     */
    public function restore(TaskRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('timesheet::task.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/timesheet/task'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/timesheet/task/'))
                ->redirect();
        }

    }

}
